CREATE TABLE states_of_goods (		-- stan towaru
	id SERIAL PRIMARY KEY,
	name text NOT NULL UNIQUE
);

CREATE TABLE kinds_of_goods (		-- rodzaj towaru, kategoria
       id SERIAL PRIMARY KEY,       
       name text NOT NULL UNIQUE
);

CREATE TABLE kinds_of_pay (	-- rodzaj platnosci 
	id SERIAL PRIMARY KEY,
	name text NOT NULL UNIQUE
);

CREATE TABLE despatch_on_cost ( --wysylka na koszt
	id SERIAL PRIMARY KEY,
	name text NOT NULL UNIQUE
);
	
-- Kind of Invoices
CREATE TABLE invoices_kind (
	id SERIAL PRIMARY KEY,
	name text NOT NULL UNIQUE
);

CREATE TABLE units (
	id SERIAL PRIMARY KEY,
	name varchar(4) NOT NULL,
	memo text
);
	
CREATE TABLE states_of_user (
	id SERIAL PRIMARY KEY,
	name text NOT NULL UNIQUE,
	access_allowed BOOLEAN DEFAULT false
);

CREATE TABLE stores (				-- magazyny
	id SERIAL PRIMARY KEY,
	name	TEXT,
	memo TEXT
);

CREATE TABLE means_of_transport (	-- srodki transportu
	id SERIAL PRIMARY KEY,
	name	TEXT,
	memo TEXT
);

CREATE TABLE users (
	id SERIAL PRIMARY KEY,
	group_id INT NOT NULL,
	fullname text, -- imie i nazwisko
	name text NOT NULL UNIQUE,
	password text NOT NULL,
	states_of_user_id INT NOT NULL,
	entry_time timestamp DEFAULT now(),
	interface text,
	ip text,
	is_logged_in BOOLEAN DEFAULT FALSE,
	FOREIGN KEY (states_of_user_id)
	REFERENCES states_of_user (id)
	ON DELETE SET NULL
	ON UPDATE CASCADE
	DEFERRABLE
	INITIALLY DEFERRED
);

CREATE TABLE clients(
	id SERIAL PRIMARY KEY,		-- version of client data
	no	INT NOT NULL,		-- no of client, identifies client
	nip varchar(16) NOT NULL,
	regon varchar(9),
	name text NOT NULL,
 	street text NOT NULL,
	postal_code varchar(6) NOT NULL,
	city text NOT NULL,
	contact_person text,
	telephone text,
	fax text,
	email text,
	www text,
	bank text, -- musimy znac bank dla przelewow
	nr_bank text, -- numer konta bankowego
	discount	INT default 0,		-- procentowy rabat
	creation_time   timestamp default now(),
	creator_id	INT NOT NULL,
--	edit_time	timestamp,
--	editor_id	INT,
	is_active	BOOLEAN default TRUE,		-- is that client active at all?
	memo text,
	FOREIGN KEY (creator_id)
	REFERENCES users (id)
	ON DELETE SET NULL
	ON UPDATE CASCADE
	DEFERRABLE
	INITIALLY DEFERRED
);

CREATE TABLE goods (				-- towary
	id SERIAL PRIMARY KEY,
	code varchar(5),		-- kod towaru
	name text NOT NULL,
	sww varchar(20),				
	units_id int NOT NULL,
	states_of_goods_id INT NOT NULL,
	stores_id INT NOT NULL,
	kinds_of_goods_id INT NOT NULL,
	memo text,
	creation_time timestamp default now(),
	creator_id	INT NOT NULL,
	edit_time	timestamp,
	editor_id	INT,
	is_active BOOLEAN default 'TRUE',
	FOREIGN KEY (stores_id)
	REFERENCES stores (id)
	ON DELETE SET NULL
	ON UPDATE CASCADE
	DEFERRABLE
	INITIALLY DEFERRED,
	FOREIGN KEY (creator_id)
	REFERENCES users (id)
	ON DELETE SET NULL
	ON UPDATE CASCADE
	DEFERRABLE
	INITIALLY DEFERRED,
	FOREIGN KEY (units_id)
	REFERENCES units (id)
	ON DELETE SET NULL
	ON UPDATE CASCADE
	DEFERRABLE
	INITIALLY DEFERRED,
	FOREIGN KEY (states_of_goods_id)
	REFERENCES states_of_goods (id)
	ON DELETE SET NULL
	ON UPDATE CASCADE
	DEFERRABLE
	INITIALLY DEFERRED,
	FOREIGN KEY (kinds_of_goods_id)
	REFERENCES kinds_of_goods (id)
	ON DELETE SET NULL
	ON UPDATE CASCADE
	DEFERRABLE
	INITIALLY DEFERRED
);

CREATE TABLE goods_data (	-- tu jest urzeczywistniona propozycja znahora
	id SERIAL PRIMARY KEY,
	goods_id	INT NOT NULL,
	no		INT NOT NULL,
	store_state FLOAT NOT NULL,
	bought_price FLOAT NOT NULL,
	sell_price FLOAT NOT NULL,
	tax_vat	INT NOT NULL,			-- stawka vat
	FOREIGN KEY (goods_id)
	REFERENCES goods (id)
	ON DELETE SET NULL
	ON UPDATE CASCADE
	DEFERRABLE
	INITIALLY DEFERRED
	
);
CREATE TABLE wz(
	id SERIAL PRIMARY KEY,
	number text UNIQUE,	
	clients_id int NOT NULL, --lepiej niech bedzie przed _ wskazuje na tabele
	sent_date date default now(),			-- wyslano dnia
	print_date date default now(),		-- data wystawienia
	orders text,	
	despatch_on_cost_id INT NOT NULL, --wysylka na koszt
	stores_id INT NOT NULL,				-- numer magazynu
	means_of_transport_id INT NOT NULL,	-- srodek transportu
	memo TEXT,
	FOREIGN KEY (means_of_transport_id)
	REFERENCES means_of_transport (id)
	ON DELETE SET NULL
	ON UPDATE CASCADE
	DEFERRABLE
	INITIALLY DEFERRED,
	FOREIGN KEY (clients_id)
	REFERENCES clients (id)
	ON DELETE SET NULL
	ON UPDATE CASCADE
	DEFERRABLE
	INITIALLY DEFERRED,
	FOREIGN KEY (stores_id)
	REFERENCES stores (id)
	ON DELETE SET NULL
	ON UPDATE CASCADE
	DEFERRABLE
	INITIALLY DEFERRED,
	FOREIGN KEY (despatch_on_cost_id)
	REFERENCES despatch_on_cost (id)
	ON DELETE SET NULL
	ON UPDATE CASCADE
	DEFERRABLE
	INITIALLY DEFERRED
);

CREATE TABLE invoices (
	id SERIAL PRIMARY KEY,
	prefix_number text,
	number text,
	orders text,	
	place text,
	nr_wz text,
	clients_id int NOT NULL,
	clients_gets_id int NOT NULL,
	sell_date date,				-- data sprzedazy
	print_date date,			-- data wystawienia
	kinds_of_pay int NOT NULL,
	term_of_pay date, -- termin platnosci
	entry_time timestamp default now(),
	users_id int NOT NULL,		-- uzytkownik wprowadzajacy
	people_get text, -- osoba odbierajaca fakture
	is_active boolean default TRUE,
	invoices_kind_id int NOT NULL,
	memo TEXT,
	FOREIGN KEY (invoices_kind_id)
	REFERENCES invoices_kind (id)
	ON DELETE SET NULL
	ON UPDATE CASCADE
	DEFERRABLE
	INITIALLY DEFERRED,
	FOREIGN KEY (kinds_of_pay)
	REFERENCES kinds_of_pay (id)
	ON DELETE SET NULL
	ON UPDATE CASCADE
	DEFERRABLE
	INITIALLY DEFERRED,
	FOREIGN KEY (clients_gets_id)
	REFERENCES clients (id)
	ON DELETE SET NULL
	ON UPDATE CASCADE
	DEFERRABLE
	INITIALLY DEFERRED,
	FOREIGN KEY (clients_id)
	REFERENCES clients (id)
	ON DELETE SET NULL
	ON UPDATE CASCADE
	DEFERRABLE
	INITIALLY DEFERRED,
	FOREIGN KEY (users_id)
	REFERENCES users (id)
	ON DELETE SET NULL
	ON UPDATE CASCADE
	DEFERRABLE
	INITIALLY DEFERRED
);

CREATE TABLE invoice_goods(
	id SERIAL PRIMARY KEY,
	invoices_id	INT NOT NULL,
	goods_id int NOT NULL,
	goods_data_id INT NOT NULL,	-- wersja, oznacza wersje cenowa towaru
	quantity float4 NOT NULL,
	entry_time timestamp default now(),
	FOREIGN KEY (goods_id)
	REFERENCES goods (id)
	ON DELETE SET NULL
	ON UPDATE CASCADE
	DEFERRABLE
	INITIALLY DEFERRED,
	FOREIGN KEY (invoices_id)
	REFERENCES invoices (id)
	ON DELETE SET NULL
	ON UPDATE CASCADE
	DEFERRABLE
	INITIALLY DEFERRED
);

CREATE TABLE packages (
	id SERIAL PRIMARY KEY,
	no INT NOT NULL,
	name text NOT NULL,
	price float8 NOT NULL,
	tax int4 NOT NULL,
	is_active BOOLEAN default 'TRUE'
);

CREATE TABLE invoice_packages (
	id SERIAL PRIMARY KEY,
	packages_id	INT,
	clients_id	INT,
	quantity_sent float8 NOT NULL,
	quantity_received float8 NOT NULL,
	entry_time timestamp default now(),
	FOREIGN KEY (clients_id)
	REFERENCES clients (id)
	ON DELETE SET NULL
	ON UPDATE CASCADE
	DEFERRABLE
	INITIALLY DEFERRED,
	FOREIGN KEY (packages_id)
	REFERENCES packages (id)
	ON DELETE SET NULL
	ON UPDATE CASCADE
	DEFERRABLE
	INITIALLY DEFERRED
);	
-- not discounts, but promotions
CREATE TABLE promotions (
	id SERIAL PRIMARY KEY,
	goods_id int NOT NULL, 
	nice_price float8 NOT NULL,
	date_from date default 'today',
	date_till date default 'tomorrow',
	is_active BOOLEAN default 'TRUE',
	FOREIGN KEY (goods_id)
	REFERENCES goods (id)
	ON DELETE SET NULL
	ON UPDATE CASCADE
	DEFERRABLE
	INITIALLY DEFERRED
);

CREATE TABLE remittance ( -- tabela przelewy
	id SERIAL PRIMARY KEY,
	clients_id int NOT NULL,
	date date, --data wyst. przelewu
	price float8 NOT NULL, --wartosc przelewu
	title text, -- tytulem
	entry_time timestamp default now(),
	users_id int NOT NULL,
	memo text,
	FOREIGN KEY (clients_id)
	REFERENCES clients (id)
	ON DELETE SET NULL
	ON UPDATE CASCADE
	DEFERRABLE
	INITIALLY DEFERRED,
	FOREIGN KEY (users_id)
	REFERENCES users (id)
	ON DELETE SET NULL
	ON UPDATE CASCADE
	DEFERRABLE
	INITIALLY DEFERRED
);

-- Discounts for goods
CREATE TABLE discounts (
	id SERIAL PRIMARY KEY,
	invoice_goods_id INT NOT NULL,
	discount INT default 0,
	FOREIGN KEY (invoice_goods_id)
	REFERENCES invoice_goods (id)
	ON DELETE SET NULL
	ON UPDATE CASCADE
	DEFERRABLE
	INITIALLY DEFERRED
);

	
--
-- Permissions system
--

-- Actions' groups 
CREATE TABLE groups_of_actions (
	id SERIAL PRIMARY KEY,
	name TEXT UNIQUE,
	memo TEXT
);

-- Actions
CREATE TABLE actions (
	id SERIAL PRIMARY KEY,
	name TEXT UNIQUE,
	memo TEXT
);

-- Groups to actions relations.
CREATE TABLE groups_to_actions (
	id SERIAL PRIMARY KEY,
	group_id INT,
	action_id INT
);

-- Users groups
CREATE TABLE users_groups(
	id SERIAL PRIMARY KEY,
	name TEXT UNIQUE,
	memo TEXT
);

-- Default users' permissions.
CREATE TABLE default_users_permissions (
	id SERIAL PRIMARY KEY,
	group_id INT,
	action_id INT	
);

--  Users' permissions.
CREATE TABLE users_permissions (
	user_id INT,
	action_id INT
);

