-- On inserting new user into users table, function and trigger ads it's
-- default permissions.

--DROP FUNCTION insert_user();
CREATE FUNCTION insert_user()
RETURNS opaque
AS 'BEGIN

		INSERT INTO users_permissions 
			SELECT NEW.id, dp.action_id
			FROM default_users_permissions dp
			WHERE dp.group_id=NEW.group_id;

	RETURN new;
	END;'
LANGUAGE 'plpgsql';

--DROP TRIGGER insert_user ON users;
CREATE TRIGGER insert_user
AFTER INSERT ON users
FOR EACH ROW EXECUTE PROCEDURE insert_user();


-- On group change of existing user, function revokes all actions
-- granted to user, and grants actions which are from new group.
--DROP FUNCTION update_user();
CREATE FUNCTION update_user()
RETURNS opaque
AS 'BEGIN
		IF NEW.group_id!=OLD.group_id THEN
			DELETE FROM users_permissions
				WHERE user_id=OLD.id 
					AND action_id IN ( SELECT action_id
											FROM default_users_permissions dp
											WHERE dp.group_id=OLD.group_id);

			INSERT INTO users_permissions 
				SELECT NEW.id, dp.action_id 
				FROM default_users_permissions dp
				WHERE dp.group_id=NEW.group_id;
		END IF;
	RETURN new;
	END;'
LANGUAGE 'plpgsql';

--DROP TRIGGER update_user ON users;
CREATE TRIGGER update_user
BEFORE UPDATE ON users
FOR EACH ROW EXECUTE PROCEDURE update_user();

-- After deleteing user, we have to clean up it's actions.
--DROP FUNCTION delete_user();
CREATE FUNCTION delete_user()
RETURNS opaque
AS 'BEGIN
		DELETE FROM users_permissions
			WHERE user_id=OLD.id; 
		RETURN old;
	END;'
LANGUAGE 'plpgsql';

--DROP TRIGGER delete_user ON users;
CREATE TRIGGER delete_user
AFTER delete ON users
FOR EACH ROW EXECUTE PROCEDURE delete_user();


-- On inserting an action we have to update user action, and grant it to
-- users, whou are in action group.
--DROP FUNCTION insert_action();
CREATE FUNCTION insert_action()
RETURNS opaque
AS 'BEGIN
		INSERT INTO users_permissions 
			SELECT u.is, NEW.action 
			FROM  users u
			WHERE NEW.group_id=u.group_id;

	RETURN new;
	END;'
LANGUAGE 'plpgsql';

--DROP TRIGGER insert_action ON default_users_permissions;
CREATE TRIGGER insert_action
AFTER INSERT ON default_users_permissions
FOR EACH ROW EXECUTE PROCEDURE insert_action();


--DROP FUNCTION update_action();
CREATE FUNCTION update_action()
RETURNS opaque
AS 'BEGIN
			DELETE FROM default_users_permissions 
				WHERE
					action_id=OLD.action_id AND group_id=OLD.group_id;
			INSERT INTO default_users_permissions
				VALUES (NEW.group_id, NEW.action_id);
	RETURN NULL;
	END;'
LANGUAGE 'plpgsql';
--DROP TRIGGER update_action ON default_users_permissions;
CREATE TRIGGER update_action
BEFORE UPDATE ON default_users_permissions
FOR EACH ROW EXECUTE PROCEDURE update_action();


--DROP FUNCTION delete_action();
CREATE FUNCTION delete_action()
RETURNS opaque
AS 'BEGIN
		DELETE FROM users_permissions
			WHERE action_id=OLD.action_id
				AND user_id IN (	SELECT id 
											FROM users 
											WHERE group_id=OLD.group_id ); 
		RETURN old;
	END;'
LANGUAGE 'plpgsql';

--DROP TRIGGER delete_action ON default_users_permissions;
CREATE TRIGGER delete_action
AFTER delete ON default_users_permissions
FOR EACH ROW EXECUTE PROCEDURE delete_action();

